using System;
using System.Collections.Generic;
using System.IO;
using javax.crypto;
using javax.crypto.spec;
using java.security;
using java.security.spec;
using Org.BouncyCastle.Crypto;
using System.Diagnostics;
using System.Security.Permissions;

namespace Proyecto
{
    /*
     * Clase CifrarArchivoYCabeceras: cifra un archivo, aadindole una lista de cabeceras
     * y una firma
     * Autor: Juan lvaro Muoz Naranjo
     * Fecha: Junio de 2007
     */
    class CifrarArchivoYCabeceras
    {
        //longitud (EN BYTES) DEBE SER POTENCIA DE 2,
        const int longitud = 8;

        //cifra un archivo, le aade la lista de cabeceras y lo firma con RSA-SHA1
        internal static void cifrarConCabeceras(string archivo_origen, string archivo_destino, Cabeceras cabeceras, string algoritmoSimetrico, byte[] vectorInicializacion, SecretKey clave, string nombreUsuario, PrivateKey clavePrivadaRSA)
        {
            FileStream fs_fuente;
            FileStream fs_destino;
            BinaryReader br_fuente;
            BinaryWriter bw_destino;
            Cipher cipher;
            Signature signature = Signature.getInstance("SHA1withRSA");
            signature.initSign(clavePrivadaRSA);
            AlgorithmParameterSpec VI;
            byte[] buffer_origen = new byte[longitud];
            byte[] buffer_destino = null;
            byte[] buffer_cabeceras = null;
            byte[] buffer_VI_cifrado = null;
            byte[] nombre_usuario_cifrado = null;
            byte[] firma = null;
            long pos_long_archivo_cifrado;
            long long_archivo_cifrado = 0;
            int bytes_leidos = 0;

            try
            {
                //abrimos el archivo fuente
                //LO ABRIMOS PRIMERO POR SI SALTA UNA EXCEPCIN, DE ESTA FORMA, SI EL ARCHIVO
                //DESTINO YA HUBIESE SIDO CREADO POR OTRO EVENTO Changed ANTERIOR, NO SE SOBREESCRIBIR
                fs_fuente = new FileStream(archivo_origen, FileMode.Open, FileAccess.Read);
                br_fuente = new BinaryReader(fs_fuente);

                //abrimos el archivo destino
                fs_destino = new FileStream(archivo_destino, FileMode.Create, FileAccess.Write);
                bw_destino = new BinaryWriter(fs_destino);

                //escribimos las cabeceras
                buffer_cabeceras = cabeceras.aBytes();
                bw_destino.Write(buffer_cabeceras);

                ////////////////////
                //NOMBRE DEL FIRMANTE CIFRADO
                ////////////////////
                //escribimos el nombre del firmante cifrado en modo ECB
                cipher = Cipher.getInstance(algoritmoSimetrico + "/ECB/PKCS5Padding");
                cipher.init(Cipher.ENCRYPT_MODE, clave);
                nombre_usuario_cifrado = cipher.doFinal(Utilidades.charsABytes(nombreUsuario.ToCharArray()));
                //escribimos primero su longitud, para leerlo al descifrar
                bw_destino.Write(nombre_usuario_cifrado.Length);
                //ahora escribimos el cifrado
                bw_destino.Write(nombre_usuario_cifrado);

                ////////////////////
                //VECTOR DE INICIALIZACIN CIFRADO
                ////////////////////
                //a continuacin escribimos primero la longitud del cifrado
                //del vector de inicializacin
                //despus el propio vector de inicializacin cifrado en modo ECB
                cipher = Cipher.getInstance(algoritmoSimetrico + "/ECB/PKCS5Padding");
                cipher.init(Cipher.ENCRYPT_MODE, clave);
                buffer_VI_cifrado = cipher.doFinal(vectorInicializacion);
                //long. VI cifrado
                bw_destino.Write(buffer_VI_cifrado.Length);
                //VI cifrado
                bw_destino.Write(buffer_VI_cifrado);

                ////////////////////
                //LONGITUD DEL ARCHIVO
                ////////////////////
                //dejamos un espacio de 8 bytes (long) para almacenar despus 
                //la longitud del archivo cifrado. Primero marcamos la posicin para
                //volver una vez calculada la longitud
                pos_long_archivo_cifrado = fs_destino.Position;
                bw_destino.Write(long_archivo_cifrado);

                ////////////////////
                //ARCHIVO ORIGINAL CIFRADO
                ////////////////////
                //leemos trozos de "longitud" bytes hasta el ltimo
                //inicializamos el cifrador en modo CBC
                VI = new IvParameterSpec(vectorInicializacion);
                cipher = Cipher.getInstance(algoritmoSimetrico + "/CBC/PKCS5Padding");
                cipher.init(Cipher.ENCRYPT_MODE, clave, VI);
                while ((bytes_leidos = br_fuente.Read(buffer_origen, 0, buffer_origen.Length)) == longitud)
                {
                    //alimentamos signature con los bites sin cifrar
                    signature.update(buffer_origen);
                    buffer_destino = cipher.update(buffer_origen); //cifrado
                    //actualizamos la longitud del archivo cifrado
                    long_archivo_cifrado += buffer_destino.Length;
                    bw_destino.Write(buffer_destino); //escribimos el cifrado
                    buffer_origen = new byte[longitud];
                }

                signature.update(buffer_origen, 0, bytes_leidos);
                buffer_destino = null;
                buffer_destino = cipher.doFinal(buffer_origen, 0, bytes_leidos);
                bw_destino.Write(buffer_destino);
                long_archivo_cifrado += buffer_destino.Length;

                bw_destino.Write("hola");

                //firmamos y escribimos la firma
                firma = signature.sign();
                bw_destino.Write(firma);
                
                //escribimos el campo que indica la longitud del archivo cifrado, justo antes de este
                bw_destino.Seek((int)pos_long_archivo_cifrado, SeekOrigin.Begin);
                bw_destino.Write(long_archivo_cifrado);

                bw_destino.Close();
                br_fuente.Close();
                fs_destino.Close();
                fs_fuente.Close();

                return;
            }
            catch (FileNotFoundException)
            {
                throw;
            }
            catch (IOException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //descifra un archivo y verifica la firma RSA-SHA1
        internal static bool descifrarConCabeceras(string archivo_origen, string archivo_destino, PrivateKey clavePrivadaRSA, AsymmetricKeyParameter clavePrivadaElGamal, Usuarios usuarios, out string firmante)
        {
            FileStream fs_fuente;
            FileStream fs_destino;
            BinaryReader br_fuente;
            BinaryWriter bw_destino;

            SecretKeySpec claveSimetrica;
            Cipher cipher;
            KeyFactory factory = KeyFactory.getInstance("RSA");
            byte[] publicaRSAFirmanteBytes;
            PublicKey publicaRSAFirmante;
            Signature signature = Signature.getInstance("SHA1withRSA");
            AlgorithmParameterSpec VI;

            Cabeceras cabeceras;
            InfoCabecera infoCabecera;

            long long_archivo_cifrado;
            int long_VI;   //long. del vector de inicializacin cifrado
            string nombreFirmante;
            int longNombreFirmanteCifrado;
            byte[] nombre_firmante_cifrado;
            byte[] buffer_origen;
            byte[] buffer_destino;
            byte[] vectorInicializacionBytesCifrado;
            byte[] vectorInicializacionBytes;
            byte[] firma;

            try
            {
                //abrimos el archivo
                fs_fuente = new FileStream(archivo_origen, FileMode.Open, FileAccess.Read);
                br_fuente = new BinaryReader(fs_fuente);
                //recuperamos las cabeceras
                cabeceras = Cabeceras.recuperarCabeceras(fs_fuente);
                //buscamos una cabecera que pueda descifrarse con la clave privada
                infoCabecera = cabeceras.buscarCabecera(clavePrivadaRSA);
                //no se encontr con RSA, probamos con El Gamal
                if (infoCabecera == null)
                {
                    infoCabecera = cabeceras.buscarCabecera(clavePrivadaElGamal);
                    //no se encontr ninguna, terminamos, el archivo no se puede descifrar
                    if (infoCabecera == null)
                    {
                        fs_fuente.Close();
                        br_fuente.Close();
                        throw new ExcepcionCabeceraNoEncontrada();
                    }
                }
                //se encontr una, obtenemos la clave secreta para descifrar el archivo
                claveSimetrica = new SecretKeySpec(infoCabecera.ClaveSimetrica, infoCabecera.Algoritmo);

                //desciframos el nombre del firmante, para inicializar el objeto signature
                //e ir alimentndole bytes
                cipher = Cipher.getInstance(infoCabecera.Algoritmo + "/ECB/PKCS5Padding");
                cipher.init(Cipher.DECRYPT_MODE, claveSimetrica);
                longNombreFirmanteCifrado = br_fuente.ReadInt32();
                nombre_firmante_cifrado = br_fuente.ReadBytes(longNombreFirmanteCifrado);
                nombreFirmante = Utilidades.bytesAString(cipher.doFinal(nombre_firmante_cifrado));
                firmante = nombreFirmante;
                publicaRSAFirmanteBytes = usuarios.buscarClavePublicaPorNombre(nombreFirmante, "RSA");
                if (publicaRSAFirmanteBytes != null)
                {
                    publicaRSAFirmante = factory.generatePublic(new X509EncodedKeySpec(publicaRSAFirmanteBytes));
                    signature.initVerify(publicaRSAFirmante);
                }

                //obtenemos la long. del cifrado del vector de inicializacin
                long_VI = br_fuente.ReadInt32(); //(es un entero)

                //obtenemos el vector de inicializacin, que est cifrado en modo ECB
                cipher = Cipher.getInstance(infoCabecera.Algoritmo + "/ECB/PKCS5Padding");
                cipher.init(Cipher.DECRYPT_MODE, claveSimetrica);
                //long_bloque = Convert.ToInt32(infoCabecera.ParamsAlgoritmo);
                vectorInicializacionBytesCifrado = br_fuente.ReadBytes(long_VI);
                vectorInicializacionBytes = cipher.doFinal(vectorInicializacionBytesCifrado);
                VI = new IvParameterSpec(vectorInicializacionBytes);

                //leemos el campo "longitud del archivo", para saber cuntos bytes tenemos que descifrar
                long_archivo_cifrado = br_fuente.ReadInt64(); //(es un long)

                //inicializamos de nuevo el descifrador, esta vez en modo CBC
                cipher = Cipher.getInstance(infoCabecera.Algoritmo + "/CBC/PKCS5Padding");
                cipher.init(Cipher.DECRYPT_MODE, claveSimetrica, VI);
                fs_destino = new FileStream(archivo_destino, FileMode.Create, FileAccess.Write);
                bw_destino = new BinaryWriter(fs_destino);

                //leemos trozos del archivo cifrado, los desciframos y los escribimos en el archivo destino
                for (int procesados = 0; procesados < long_archivo_cifrado - longitud; procesados += longitud)
                {
                    //leemos del archivo origen
                    buffer_origen = br_fuente.ReadBytes(longitud);
                    buffer_destino = cipher.update(buffer_origen); //desciframos
                    //alimentamos signature con los bytes descifrados
                    if (publicaRSAFirmanteBytes != null)
                        signature.update(buffer_destino);
                    //escribimos en el archivo destino
                    bw_destino.Write(buffer_destino);
                }
                //ltimo trozo
                buffer_origen = br_fuente.ReadBytes(longitud);
                buffer_destino = cipher.doFinal(buffer_origen);
                if (publicaRSAFirmanteBytes != null)
                    signature.update(buffer_destino);
                bw_destino.Write(buffer_destino);

                br_fuente.ReadString();

                //leemos la firma al final del archivo
                firma = br_fuente.ReadBytes((int)(fs_fuente.Length - fs_fuente.Position));
                
                //cerrramos todo
                fs_fuente.Close();
                br_fuente.Close();
                fs_destino.Close();
                bw_destino.Close();

                //verificamos la firma
                if (publicaRSAFirmanteBytes == null)
                    throw new ExcepcionFirmaNoVerificada(nombreFirmante);
                else
                    return signature.verify(firma);
            }
            catch (FileNotFoundException)
            {
                throw;
            }
            catch (IOException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }       
    }
}