using System;
using System.Collections.Generic;
using System.Runtime.Serialization.Formatters.Binary;
using System.IO;
using System.Text;
using javax.crypto;
using java.security;

namespace Proyecto
{
    /*
     * Clase EstructuraClaveSistema: estructuras almacenadas en el archivo "listaClaveSistema.lcs"
     * Autor: Juan lvaro Muoz Naranjo
     * Fecha: Junio de 2007
     */
    [Serializable]
    class EstructuraClaveSistema
    {
        private string nombreUsuario;   //nombre del usuario
        private byte[] claveSistema;    //la clave simtrica que usa el sistema para cifrar sus archivos

        internal EstructuraClaveSistema(string nombreUsuario, byte[] claveSistema)
        {
            this.nombreUsuario = nombreUsuario;
            this.claveSistema = claveSistema;
        }

        internal EstructuraClaveSistema(string nombreUsuario, SecretKey claveSistema)
        {
            this.nombreUsuario = nombreUsuario;
            this.claveSistema = claveSistema.getEncoded();
        }

        internal string NombreUsuario
        {
            get
            {
                return nombreUsuario;
            }
        }

        internal byte[] ClaveSistema
        {
            get
            {
                return claveSistema;
            }
        }

        //serializa el objeto en un vector de bytes
        internal byte[] aBytes()
        {
            MemoryStream memStream = new MemoryStream();
            BinaryWriter binWriter = new BinaryWriter(memStream);

            binWriter.Write(nombreUsuario);
            binWriter.Write(claveSistema);

            binWriter.Close();
            memStream.Close();

            return memStream.ToArray();
        }

        //serializa this y cifra el resultado con una clave pblica RSA
        internal byte[] cifrar(string algoritmo, PublicKey clavePublica)
        {
            Cipher cipher = Cipher.getInstance(algoritmo);
            cipher.init(Cipher.ENCRYPT_MODE, clavePublica);

            try
            {
                return cipher.doFinal(this.aBytes());
            }
            catch (Exception e)
            {
                Console.WriteLine("CabeceraClaveSistema.cifrar(): " + e.Message);
                return null;
            }
        }

        //descifra con una clave privada RSA y deserializa
        internal static EstructuraClaveSistema recuperarCabeceraClaveSistema(byte[] bytes,
            string algoritmo, PrivateKey clavePrivada)
        {
            MemoryStream memStream;
            BinaryReader binReader;

            Cipher cipher = Cipher.getInstance(algoritmo);
            cipher.init(Cipher.DECRYPT_MODE, clavePrivada);

            string nombre;
            byte[] clave;

            //desciframos
            memStream = new MemoryStream(cipher.doFinal(bytes));
            binReader = new BinaryReader(memStream);
            //recuperamos la informacin
            nombre = binReader.ReadString();
            clave = binReader.ReadBytes((int)memStream.Length - nombre.Length);

            return new EstructuraClaveSistema(nombre, clave);
        }
    }
}
