using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Text;
using System.Windows.Forms;
using java.security;
using java.security.spec;
using Org.BouncyCastle.Crypto;

namespace Proyecto
{
    /*
     * Clase FormModificarUsuario: muestra una ventana que permite modificar un usuario
     * Autor: Juan lvaro Muoz Naranjo
     * Fecha: Junio de 2007
     */
    public partial class FormModificarUsuario : Form
    {
        public FormModificarUsuario(ProgressBar barraInvocador, System.Windows.Forms.Timer temp)
        {
            InitializeComponent();
            this.Text = Datos.NombreUsuario;
            this.barraInvocador = barraInvocador;
            this.temporizador = temp;
        }

        //invocado cuando el usuario pulsa el botn Aceptar
        private void bAceptar_Click(object sender, EventArgs e)
        {
            if (temporizador != null) temporizador.Stop();

            Usuario usuarioActual = null;
            Usuario usuarioNuevo = null;
            string nombreNuevo = null;
            string emailNuevo = null;
            KeyPair parRSANuevo = null;
            AsymmetricCipherKeyPair parElGamalNuevo = null;
            ParClavePublicaNombreAlgoritmo parPublicaRSANuevo = null;
            ParClavePublicaNombreAlgoritmo parPublicaElGamalNuevo = null;

            try
            {
                //buscamos el usuario en la lista de usuarios del sistema
                usuarioActual = Datos.Usuarios.buscarUsuarioPorNombre(Datos.NombreUsuario);

                barraInvocador.Increment(10);
                if (tbNombre.Text == "")
                    nombreNuevo = Datos.NombreUsuario;
                else
                {
                    if (Datos.Usuarios.buscarUsuarioPorNombre(tbNombre.Text) != null)
                    {
                        MessageBox.Show("El nuevo nombre de usuario ya est en uso. Elija otro.");
                        return;
                    }
                    nombreNuevo = tbNombre.Text;
                }

                if (tbEmail.Text == "")
                    emailNuevo = usuarioActual.Email;
                else
                    emailNuevo = tbEmail.Text;

                if (tbFrase.Text != "" && tbFraseNueva.Text != "" && tbConfirmeFraseNueva.Text != "")
                {
                    //la frase de acceso y su confirmacin coinciden
                    if (tbFraseNueva.Text == tbConfirmeFraseNueva.Text)
                    {
                        //si la frase de acceso es correcta
                        if (Datos.ClavePrivadaRSAUsuario.Equals(Utilidades.generarParDeClavesRSAconPBE(1024, tbFrase.Text.ToCharArray()).getPrivate()))
                        {
                            parRSANuevo = Utilidades.generarParDeClavesRSAconPBE(1024, tbFraseNueva.Text.ToCharArray());
                            barraInvocador.Increment(20);
                            parElGamalNuevo = Utilidades.generarParDeClavesElGamalconPBE(512, tbFraseNueva.Text.ToCharArray());
                            barraInvocador.Increment(30);
                            parPublicaRSANuevo = new ParClavePublicaNombreAlgoritmo("RSA", parRSANuevo.getPublic().getEncoded());
                            parPublicaElGamalNuevo = new ParClavePublicaNombreAlgoritmo("ElGamal", Utilidades.codificarClaveElGamal(parElGamalNuevo.Public));
                        }
                    } //no coinciden
                    else
                    {
                        MessageBox.Show("La nueva frase de acceso y su confirmacin no coinciden");
                        tbFraseNueva.Clear();
                        tbConfirmeFraseNueva.Clear();
                        if (temporizador != null) temporizador.Start();
                        return;
                    }
                }
                //alguno de los campos de frase no est relleno
                else if (!(tbFrase.Text == "" && tbFraseNueva.Text == "" && tbConfirmeFraseNueva.Text == ""))
                {
                    MessageBox.Show("Rellene los tres ltimos campos si desea modificar su frase de acceso.");
                    tbFrase.Clear();
                    tbFraseNueva.Clear();
                    tbConfirmeFraseNueva.Clear();
                    if (temporizador != null) temporizador.Start();
                    return;
                }

                barraInvocador.Increment(10);
                //instanciamos el nuevo usuario
                if (parPublicaRSANuevo == null && parElGamalNuevo == null)
                    usuarioNuevo = new Usuario(nombreNuevo, emailNuevo, true,
                        new ParClavePublicaNombreAlgoritmo("RSA", usuarioActual.buscarClavePublica("RSA")),
                        new ParClavePublicaNombreAlgoritmo("ElGamal", usuarioActual.buscarClavePublica("ElGamal")));
                else if (parPublicaRSANuevo != null && parPublicaElGamalNuevo != null)
                    usuarioNuevo = new Usuario(nombreNuevo, emailNuevo, true, parPublicaRSANuevo, parPublicaElGamalNuevo);
                else
                {
                    MessageBox.Show("Error en las claves. Intntelo de nuevo.");
                    if (temporizador != null) temporizador.Start();
                    return;
                }

                //sustituimos el usuario en la lista de usuarios y la lista de claves del sistema
                Datos.Usuarios.eliminarUsuarioPorNombre(Datos.NombreUsuario);
                Datos.Usuarios.aadirUsuario(usuarioNuevo);
                Datos.Usuarios.guardarEnArchivoCifrado("usuarios.usr.temp", "DES/ECB/PKCS5Padding", Datos.ClaveSistema);

                barraInvocador.Increment(30);
                ListaClaveSistema listaClaveSistema = new ListaClaveSistema("listaClaveSistema.lcs");
                KeyFactory factory = KeyFactory.getInstance("RSA");
                PublicKey publicaRSA = factory.generatePublic(new X509EncodedKeySpec(usuarioNuevo.buscarClavePublica("RSA")));
                listaClaveSistema.eliminarEstructuraClaveSistema(Datos.NombreUsuario, "RSA", Datos.ClavePrivadaRSAUsuario);
                listaClaveSistema.aadirEstructuraClaveSistema(new EstructuraClaveSistema(usuarioNuevo.Nombre,Datos.ClaveSistema.getEncoded()),
                    "RSA",publicaRSA);
                barraInvocador.Increment(30);
                listaClaveSistema.guardarListaEnArchivo("listaClaveSistema.lcs.temp");

                File.Replace("usuarios.usr.temp", "usuarios.usr", "usuarios.usr.old");
                File.Replace("listaClaveSistema.lcs.temp", "listaClaveSistema.lcs", "listaClaveSistema.lcs.old");
                barraInvocador.Increment(10);
                //finalmente, actualizamos los campos de la clase Datos
                Datos.NombreUsuario = usuarioNuevo.Nombre;
                if(parRSANuevo!=null)
                    Datos.ClavePrivadaRSAUsuario = parRSANuevo.getPrivate();
                barraInvocador.Increment(50);
            }
            catch(Exception exc)
            {
                if (temporizador != null) temporizador.Start();
                MessageBox.Show(exc.Message);
            }
            barraInvocador.Value = 0;
            if (temporizador != null) temporizador.Start();
            this.Close();
        }

        //invocado cuando el usuario pulsa el botn Cancelar
        private void bCancelar_Click(object sender, EventArgs e)
        {
            if (temporizador != null)
            {
                temporizador.Stop();
                temporizador.Start();
            }
            this.Close();
        }
    }
}