using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Windows.Forms;

namespace Proyecto
{
    /*
     * Clase FormPedirParametrosCifrado: muestra una ventana que pide
     * al usuario los distintos parmetros para el cifrado del archivo: usuarios
     * con permiso, algoritmo simtrico (y longitud de clave) y asimtrico
     * Autor: Juan lvaro Muoz Naranjo
     * Fecha: Junio de 2007
     */
    public partial class FormPedirParametrosCifrado : Form
    {
        public FormPedirParametrosCifrado(string nombreArchivo, System.Windows.Forms.Timer temp)
        {
            InitializeComponent();
            this.Text = nombreArchivo;
            this.nombresUsuariosPermiso = new ArrayList();
            this.temporizador = temp;
        }

        //al elegir un algoritmo simtrico
        private void cbAlgSimetrico_SelectionChangeCommitted(object sender, EventArgs e)
        {
            if (temporizador != null)
            {
                temporizador.Stop();
                temporizador.Start();
            }
            if ((string)cbAlgSimetrico.SelectedItem == "DES") //DES
            {
                algoritmoSimetrico = "DES";
                cbLongClaveSimetrica.Enabled = false;
            }
            else if ((string)cbAlgSimetrico.SelectedItem == "DES-EDE") //DES-EDE
            {
                algoritmoSimetrico = "DESede";
                cbLongClaveSimetrica.Enabled = false;
            }
            else if ((string)cbAlgSimetrico.SelectedItem == "AES") //AES
            {
                algoritmoSimetrico = "AES";
                cbLongClaveSimetrica.Items.Clear();
                cbLongClaveSimetrica.Items.AddRange(new object[] { "128", "192", "256" });
                cbLongClaveSimetrica.SelectedIndex = 0;
                longClaveSimetrica = 128;
                cbLongClaveSimetrica.Enabled = true;
            }
            else if ((string)cbAlgSimetrico.SelectedItem == "Blowfish") //Blowfish
            {
                algoritmoSimetrico = "Blowfish";
                cbLongClaveSimetrica.Items.Clear();
                cbLongClaveSimetrica.Items.AddRange(new object[] { "128", "320" });
                cbLongClaveSimetrica.SelectedIndex = 0;
                longClaveSimetrica = 128;
                cbLongClaveSimetrica.Enabled = true;
            }
        }

        //al elegir una longitud de clave
        private void cbLongClaveSimetrica_SelectionChangeCommitted(object sender, EventArgs e)
        {
            if (temporizador != null)
            {
                temporizador.Stop();
                temporizador.Start();
            }
            if ((string)cbLongClaveSimetrica.SelectedItem == "128")
                longClaveSimetrica = 128;
            else if ((string)cbLongClaveSimetrica.SelectedItem == "192")
                longClaveSimetrica = 192;
            else if ((string)cbLongClaveSimetrica.SelectedItem == "256")
                longClaveSimetrica = 256;
            else if ((string)cbLongClaveSimetrica.SelectedItem == "320")
                longClaveSimetrica = 320;
        }

        //invocado cuando el usuario pulsa el botn Aceptar
        private void bAceptar_Click(object sender, EventArgs e)
        {
            if (temporizador != null)
            {
                temporizador.Stop();
                temporizador.Start();
            }
            if (lbUsuariosPermiso.Items.Count == 0)
            {
                MessageBox.Show("Elija algn usuario para los permisos del archivo.");
                return;
            }
            foreach (Object o in lbUsuariosPermiso.Items)
                nombresUsuariosPermiso.Add((string)o);

            this.resultado = DialogResult.OK;
            this.Close();
        }

        //invocado antes de mostrar la ventana
        private void FormPedirParametrosCifrado_Load(object sender, EventArgs e)
        {
            //colocamos todos los usuarios, excepto el nuestro, en lbUsuarios, y
            //colocamos el nuestro en la lbUsuariosPermiso
            lbUsuarios.Items.Clear();
            for (int i = 0; i < Datos.Usuarios.tamao(); i++)
                lbUsuarios.Items.Add(Datos.Usuarios.buscarUsuario(i).Nombre);
            lbUsuariosPermiso.Items.Clear();
            lbUsuarios.Items.Remove((string)Datos.NombreUsuario);
            lbUsuariosPermiso.Items.Add(Datos.NombreUsuario);

            cbAlgSimetrico.SelectedItem = "DES";
            algoritmoSimetrico = "DES";
            cbAlgAsimetrico.SelectedItem = "RSA";
            algoritmoAsimetrico = "RSA";
        }

        //al elegir un algoritmo de clave pblica
        private void cbAlgAsimetrico_SelectionChangeCommitted(object sender, EventArgs e)
        {
            if (temporizador != null)
            {
                temporizador.Stop();
                temporizador.Start();
            }
            if ((string)cbAlgAsimetrico.SelectedItem == "RSA")
                algoritmoAsimetrico = "RSA";
            else if ((string)cbAlgAsimetrico.SelectedItem == "El Gamal")
                algoritmoAsimetrico = "ElGamal";
        }

        //al pulsar el botn ->
        private void bDerecha_Click(object sender, EventArgs e)
        {
            if (temporizador != null)
            {
                temporizador.Stop();
                temporizador.Start();
            }
            string nombreUsuario;

            if (lbUsuarios.SelectedIndex > -1)
            {
                nombreUsuario = (string)lbUsuarios.SelectedItem;
                lbUsuariosPermiso.Items.Add(nombreUsuario);
                lbUsuarios.Items.Remove(nombreUsuario);
            }
        }

        //al pulsar el botn <-
        private void bIzquierda_Click(object sender, EventArgs e)
        {
            if (temporizador != null)
            {
                temporizador.Stop();
                temporizador.Start();
            }
            string nombreUsuario;
            if (lbUsuariosPermiso.SelectedIndex > -1)
            {
                nombreUsuario = (string)lbUsuariosPermiso.SelectedItem;
                lbUsuarios.Items.Add(nombreUsuario);
                lbUsuariosPermiso.Items.Remove(nombreUsuario);
            }
        }

        //invocado cuando el usuario pulsa el botn Cancelar
        private void bCancelar_Click(object sender, EventArgs e)
        {
            if (temporizador != null)
            {
                temporizador.Stop();
                temporizador.Start();
            }
            this.resultado = DialogResult.Cancel;
            this.Close();
        }
    }
}