using System;
using System.IO;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
//using System.Text;
using System.Windows.Forms;

namespace Proyecto
{
    /*
     * Clase FormUsuarios: muestra una ventana desde la que gestionar
     * los usuarios de la aplicacin
     * Autor: Juan lvaro Muoz Naranjo
     * Fecha: Junio de 2007
     */
    public partial class FormUsuarios : Form
    {
        internal FormUsuarios(System.Windows.Forms.Timer temp)
        {
            InitializeComponent();
            temporizador = temp;
        }

        //invocado al pulsar el botn Nuevo
        private void bNuevo_Click(object sender, EventArgs e)
        {
            if (temporizador != null)
            {
                temporizador.Stop();
                temporizador.Start();
            }
            FormCrearUsuario formCrearUsuario = new FormCrearUsuario(barraProgreso, temporizador);
            formCrearUsuario.ShowDialog();
        }

        //invocado al activar el formulario
        private void FormUsuarios_Activated(object sender, EventArgs e)
        {
            if (temporizador != null)
            {
                temporizador.Stop();
                temporizador.Start();
            }
            lbUsuarios.Items.Clear();
            lbUsuarios.Items.AddRange(Datos.Usuarios.nombresUsuarios());
            if ((string)lbUsuarios.SelectedItem == Datos.NombreUsuario)
                bModificar.Enabled = true;
            else
                bModificar.Enabled = false;
        }

        //invocado al pulsar el botn Eliminar
        private void bEliminar_Click(object sender, EventArgs e)
        {
            if (temporizador != null) temporizador.Stop();
            int indiceUsuario = lbUsuarios.SelectedIndex;
            string nombre = (string)lbUsuarios.SelectedItem;
            if (indiceUsuario > -1)
            {
                DialogResult dialogResult = MessageBox.Show("Desea eliminar el usuario " + (string)lbUsuarios.SelectedItem + "?",
                    "", MessageBoxButtons.YesNo);
                if (dialogResult == DialogResult.Yes)
                {
                    barraProgreso.Increment(10);
                    //eliminamos el usuario de la lista en Datos
                    Datos.Usuarios.eliminarUsuarioPorNombre(nombre);
                    barraProgreso.Increment(10);
                    //guardamos la lista en un archivo auxiliar
                    Datos.Usuarios.guardarEnArchivoCifrado("usuarios.usr.temp", "DES/ECB/PKCS5Padding", Datos.ClaveSistema);
                    barraProgreso.Increment(10);
                    //eliminamos el usuarios de la lista listaClaveSistema
                    ListaClaveSistema listaClaveSistema = new ListaClaveSistema("listaClaveSistema.lcs");
                    listaClaveSistema.eliminarEstructuraClaveSistema(indiceUsuario);
                    barraProgreso.Increment(20);
                    listaClaveSistema.guardarListaEnArchivo("listaClaveSistema.lcs.temp");
                    barraProgreso.Increment(20);
                    //sustituimos los archivos
                    File.Replace("usuarios.usr.temp", "usuarios.usr", "usuarios.usr.old");
                    File.Replace("listaClaveSistema.lcs.temp", "listaClaveSistema.lcs", "listaClaveSistema.lcs.old");
                    barraProgreso.Increment(20);
                    //actualizamos la lista que se muestra en la ventana
                    lbUsuarios.Items.RemoveAt(indiceUsuario);
                    barraProgreso.Increment(10);
                }
            }
            else
            {
                if (temporizador != null) temporizador.Start();
                MessageBox.Show("Seleccione un usuario.");
            }
            if (temporizador != null) temporizador.Start();
            barraProgreso.Value = 0;
        }

        //invocado al pulsar el botn Ver
        private void bVer_Click(object sender, EventArgs e)
        {
            if (temporizador != null)
            {
                temporizador.Stop();
                temporizador.Start();
            }
            if (lbUsuarios.SelectedIndex > -1)
            {
                Usuario usuario = Datos.Usuarios.buscarUsuarioPorNombre((string)lbUsuarios.SelectedItem);
                if(usuario.Local)
                    MessageBox.Show("Email: " + usuario.Email + "\nN claves: " + usuario.NumeroClaves + "\nUsuario local.",usuario.Nombre,MessageBoxButtons.OK,MessageBoxIcon.Information);
                else
                    MessageBox.Show("Email: " + usuario.Email + "\nN claves: " + usuario.NumeroClaves + "\nUsuario externo.", usuario.Nombre, MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            else
                MessageBox.Show("Seleccione un usuario.");
        }

        //invocado al pulsar el botn Salir
        private void bSalir_Click(object sender, EventArgs e)
        {
            if (temporizador != null)
            {
                temporizador.Stop();
                temporizador.Start();
            }
            this.Close();
        }

        //invocado al pulsar el botn Modificar
        private void bModificar_Click_1(object sender, EventArgs e)
        {
            if (temporizador != null)
            {
                temporizador.Stop();
                temporizador.Start();
            }
            if ((string)lbUsuarios.SelectedItem == Datos.NombreUsuario)
            {
                FormModificarUsuario formModificarUsuario = new FormModificarUsuario(this.barraProgreso, temporizador);
                formModificarUsuario.ShowDialog();
            }
        }

        //invocado cuando se selecciona un usuario distinto en la lista de usuarios
        private void lbUsuarios_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (temporizador != null)
            {
                temporizador.Stop();
                temporizador.Start();
            }
            if ((string)lbUsuarios.SelectedItem == Datos.NombreUsuario)
                bModificar.Enabled = true;
            else
                bModificar.Enabled = false;
        }

        //invocado al pulsar el botn Exportar
        private void bExportar_Click(object sender, EventArgs e)
        {
            if (temporizador != null) temporizador.Stop();
            if (lbUsuarios.SelectedIndex > -1)
            {
                Usuario usuario = Datos.Usuarios.buscarUsuarioPorNombre((string)lbUsuarios.SelectedItem);
                if (usuario.exportarUsuario(usuario.Nombre + ".usr", true) == false)
                {
                    if (temporizador != null) temporizador.Start();
                    MessageBox.Show("Se produjo un error. No se export el usuario.");
                }
                else
                {
                    if (temporizador != null) temporizador.Start();
                    MessageBox.Show("Usuario exportado al archivo " + usuario.Nombre + ".usr.");
                }
            }
            else
            {
                if (temporizador != null) temporizador.Start();
                MessageBox.Show("Seleccione un usuario.");
            }
            if (temporizador != null) temporizador.Start();
        }

        //invocado al pulsar el botn Importar
        private void bImportar_Click(object sender, EventArgs e)
        {
            if (temporizador != null) temporizador.Stop();
            try
            {
                if (abrirArchivoDialog.ShowDialog() == DialogResult.OK)
                {
                    Usuario nuevoUsuario = Usuario.importarUsuario(abrirArchivoDialog.FileName);
                    if (nuevoUsuario == null)
                    {
                        if (temporizador != null) temporizador.Start();
                        MessageBox.Show("Se produjo un error. No se import el usuario.");
                        return;
                    }

                    Datos.Usuarios.aadirUsuario(nuevoUsuario);
                    Datos.Usuarios.guardarEnArchivoCifrado("usuarios.usr.temp", "DES/ECB/PKCS5Padding",Datos.ClaveSistema);
                    File.Replace("usuarios.usr.temp", "usuarios.usr", "usuarios.usr.old");
                    File.Delete("usuarios.usr.temp");
                    if (temporizador != null) temporizador.Start();
                    MessageBox.Show("Usuario "+nuevoUsuario.Nombre+" importado.");
                    lbUsuarios.Refresh();
                }
            }
            catch(Exception exc)
            {
                temporizador.Start();
                MessageBox.Show(exc.Message);
            }
            if (temporizador != null) temporizador.Start();
        }
    }
}