using System;
using System.Collections.Generic;
using System.Runtime.Serialization.Formatters.Binary;
using System.IO;
using System.Collections;
using javax.crypto;
using javax.crypto.spec;
using java.security;

namespace Proyecto
{
    /*
     * Clase ListaClaveSistema: Almacena y gestiona una lista de objetos EstructuraClaveSistema, pero en forma
     * de byte[] cifrado (con la clave pblica del usuario correspondiente). Lo que realmente contiene la 
     * lista son byte[], que hay que descifrar con una clave privada y convertir a EstructuraClaveSistema
     * Autor: Juan lvaro Muoz Naranjo
     * Fecha: Junio de 2007
     */
    [Serializable]
    class ListaClaveSistema
    {
        private ArrayList lista;

        internal ListaClaveSistema()
        {
            lista = new ArrayList();
        }

        //recupera la lista de un archivo
        internal ListaClaveSistema(string nombreArchivo)
        {
            FileStream fs;
            BinaryFormatter formatter = new BinaryFormatter();

            try
            {
                fs = new FileStream(nombreArchivo, FileMode.Open, FileAccess.Read);

                lista = (ArrayList)formatter.Deserialize(fs);

                fs.Close();
            }
            catch (Exception e)
            {
                Console.WriteLine("ListaClaveSistema.recuperarListaDeArchivo: " + e.Message);

                throw;
            }            
        }

        internal int Tamao
        {
            get
            {
                return lista.Count;
            }
        }
        
        //cifra una estructura con RSA y la aade a la lista
        internal void aadirEstructuraClaveSistema(string nombreUsuario, SecretKey claveSistema, string algoritmo, PublicKey clavePublica)
        {
            if (lista == null)
                lista = new ArrayList();

            lista.Add((new EstructuraClaveSistema(nombreUsuario,claveSistema)).cifrar(algoritmo,clavePublica));
        }

        //cifra una estructura con RSA y la aade a la lista
        internal void aadirEstructuraClaveSistema(EstructuraClaveSistema estructura, string algoritmoAsimetrico, PublicKey clavePublica)
        {
            if (lista == null)
                lista = new ArrayList();
                
            lista.Add(estructura.cifrar(algoritmoAsimetrico, clavePublica));        
        }

        //busca una cabecera que pueda descifrarse con la clave privada proporcionada y cuyo
        //coincida con nombreUsuario
        internal SecretKey buscarEstructuraClaveSistema(string nombreUsuario, string algoritmo, PrivateKey clavePrivada)
        {
            Cipher cipher = Cipher.getInstance(algoritmo);
            cipher.init(Cipher.DECRYPT_MODE, clavePrivada);

            EstructuraClaveSistema cabecera;

            foreach (Object o in lista)
            {
                try
                {
                    //obtenemos el byte[] de la lista, lo desciframos y lo convertimos a CabeceraClaveSistema
                    cabecera = EstructuraClaveSistema.recuperarCabeceraClaveSistema((byte[])o,algoritmo,clavePrivada);

                    if (cabecera.NombreUsuario.Equals(nombreUsuario))
                        return new SecretKeySpec(cabecera.ClaveSistema,"DES");
                }
                catch (Exception e)
                {
                    Console.WriteLine("buscarEstructuraClaveSistema: " + e.Message);
                }
            }
            return null;
        }

        //elimina la cabecera correspondiente a nombreUsuario
        //para cuando se elimine un usuario del sistema
        internal Boolean eliminarEstructuraClaveSistema(string nombreUsuario, string algoritmo, PrivateKey clavePrivada)
        {
            Cipher cipher = Cipher.getInstance(algoritmo);
            cipher.init(Cipher.DECRYPT_MODE, clavePrivada);

            EstructuraClaveSistema cabecera;

            foreach (Object o in lista)
            {
                try
                {
                    //obtenemos el byte[] de la lista, lo desciframos y lo convertimos a CabeceraClaveSistema
                    cabecera = EstructuraClaveSistema.recuperarCabeceraClaveSistema((byte[])o, algoritmo, clavePrivada);

                    if (cabecera.NombreUsuario.Equals(nombreUsuario))
                    {
                        lista.Remove(o);
                        return true;
                    }
                }
                catch (Exception e)
                {
                    Console.WriteLine("eliminarEstructuraClaveSistema: " + e.Message);
                }                
            }
            return false;            
        }

        //elimina la cabecera situada en la posicin correspondiente
        //al ndice introducido
        internal Boolean eliminarEstructuraClaveSistema(int posicion)
        {
            if (posicion > -1 && posicion < lista.Count)
            {
                lista.RemoveAt(posicion);
                return true;
            }
            else 
                return false;                
        }

        //serializa la lista en un archivo
        internal Boolean guardarListaEnArchivo(string nombreArchivo)
        {
            FileStream fs;
            BinaryFormatter formatter = new BinaryFormatter();

            try
            {
                fs = new FileStream(nombreArchivo, FileMode.Create, FileAccess.Write);

                formatter.Serialize(fs, this.lista);

                fs.Close();

                return true;
            }
            catch (Exception)
            {
                return false;
            }
        }
    }
}
