using System;
using System.Collections;
using System.Runtime.Serialization.Formatters.Binary;
//using System.Runtime.Serialization;
//using System.Text;
using System.IO;
//using javax.crypto;
using java.security;

namespace Proyecto
{
    /*
     * Clase Usuario: almacena la informacin de un usuario
     * Autor: Juan lvaro Muoz Naranjo
     * Fecha: Junio de 2007
     */
    [Serializable]
    class Usuario
    {
        private string nombre;
        private string email;
        private bool local; //true = puede usar la aplicacin; false = slo tenemos sus clave pblicas para enviarle archivos
        private ArrayList paresClaveAlgoritmo;   //guarda una lista de objetos ParClavePublicaNombreAlgoritmo

        internal Usuario(string nombre, string email, bool pleno, params ParClavePublicaNombreAlgoritmo[] claves)
        {
            this.nombre = nombre;
            this.email = email;
            this.local = pleno;
            this.paresClaveAlgoritmo = new ArrayList();
            for (int i = 0; i < claves.Length; i++)
                this.paresClaveAlgoritmo.Add(claves[i]);
        }

        internal Usuario(string nombre, string email, bool pleno, ArrayList paresClavePublicaAlgoritmo)
        {
            this.nombre = nombre;
            this.email = email;
            this.local = pleno;
            this.paresClaveAlgoritmo = paresClavePublicaAlgoritmo;
        }

        public string Nombre
        {
            get
            {
                return nombre;
            }
        }

        public string Email
        {
            get
            {
                return email;
            }
        }

        public bool Local
        {
            get
            {
                return local;
            }
        }

        public int NumeroClaves
        {
            get
            {
                return paresClaveAlgoritmo.Count;
            }
        }

        //aade una clave pblica al usuario this
        internal void aadirClavePublica(PublicKey clavePublica)
        {
            paresClaveAlgoritmo.Add(new ParClavePublicaNombreAlgoritmo(clavePublica));
        }

        //busca la clave pblica correspondiente al algoritmo indicado
        internal byte[] buscarClavePublica(string algoritmo)
        {
            ParClavePublicaNombreAlgoritmo par;

            foreach (Object o in paresClaveAlgoritmo)
            {
                par = (ParClavePublicaNombreAlgoritmo)o;
                if (par.NombreAlgoritmo.Equals(algoritmo,StringComparison.CurrentCultureIgnoreCase))
                    return par.ClavePublica;
            }
            return null;
        }

        //elimina la clave pblica correspondiente al algoritmo indicado
        internal void eliminarClavePublica(string algoritmo)
        {
            foreach (Object o in paresClaveAlgoritmo)
            {
                if (((ParClavePublicaNombreAlgoritmo)o).NombreAlgoritmo.Equals(algoritmo, StringComparison.CurrentCultureIgnoreCase))
                {
                    paresClaveAlgoritmo.Remove(o);
                    return;
                }
            }
        }

        //serializa este objeto en un vector de bytes
        internal byte[] aBytes()
        {
            MemoryStream memStream = new MemoryStream();
            BinaryFormatter formatter = new BinaryFormatter();

            formatter.Serialize(memStream, this.nombre);
            formatter.Serialize(memStream, this.email);
            formatter.Serialize(memStream, this.local);
            formatter.Serialize(memStream, this.paresClaveAlgoritmo);

            memStream.Close();
            return memStream.ToArray();
        }

        internal static Usuario recuperarUsuario(byte[] bytes)
        {
            MemoryStream memStream = new MemoryStream(bytes);
            BinaryFormatter formatter = new BinaryFormatter();

            string nombre;
            string email;
            bool pleno;
            ArrayList claves;

            nombre = (string)formatter.Deserialize(memStream);
            email = (string)formatter.Deserialize(memStream);
            pleno = (bool)formatter.Deserialize(memStream);
            claves = (ArrayList)formatter.Deserialize(memStream);

            return new Usuario(nombre, email, pleno, claves);
        }

        //serializa el objeto this en un archivo
        internal bool exportarUsuario(string archivo, bool sobreescribir)
        {
            try
            {
                FileStream fs;
                if (sobreescribir == true)
                    fs = new FileStream(archivo, FileMode.Create);
                else
                    fs = new FileStream(archivo, FileMode.CreateNew);
                byte[] usuario_bytes = this.aBytes();
                fs.Write(usuario_bytes,0,usuario_bytes.Length);
                fs.Close();
                return true;
            }
            catch
            {
                return false;
            }
        }

        //deserializa un usuario de un archivo
        internal static Usuario importarUsuario(string archivo)
        {
            try
            {
                FileStream fs = new FileStream(archivo, FileMode.Open);
                byte[] usuario_bytes = new byte[fs.Length];
                if (fs.Read(usuario_bytes, 0, (int)fs.Length) != fs.Length)
                {
                    return null;
                }
                else
                {
                    Usuario usuario = recuperarUsuario(usuario_bytes);
                    usuario.local = false;
                    return usuario;
                }
            }
            catch
            {
                return null;
            }
        }
    }
}