using System;
using System.Collections;
using System.Runtime.Serialization.Formatters.Binary;
using System.IO;
//using System.Text;
using System.Windows.Forms;
using javax.crypto;
using java.security;
using Org.BouncyCastle.Crypto;

namespace Proyecto
{
    /*
     * Clase UsuarioS: contenedor para una lista de objetos Usuario.
     * Adems de almacenarla, se encarga de gestionarla, serializarla y deserializarla
     * de un archivo, y de cifrar y descrifrar ese mismo archivo.
     * El archivo cifrado guarda la lista (ArrayList), no el objeto Usuarios
     * Autor: Juan lvaro Muoz Naranjo
     * Fecha: Junio de 2007
     */
    public class Usuarios
    {
        private ArrayList lista;

        internal Usuarios()
        {
            this.lista = new ArrayList();
        }

        internal Usuarios(ArrayList lista)
        {
            this.lista = lista;
        }

        internal Usuarios(params Usuario[] usuarios)
        {
            lista = new ArrayList();

            for (int i = 0; i < usuarios.Length; i++)
                lista.Add(usuarios[i]);
        }

        //recupera la lista de objetos Usuario del archivo cifrado
        internal Usuarios(string archivo, string algoritmo, SecretKey clave)
        {
            FileStream fs; 
            MemoryStream memStream = new MemoryStream();
            BinaryFormatter formatter = new BinaryFormatter();
            Cipher cipher = Cipher.getInstance(algoritmo);
            cipher.init(Cipher.DECRYPT_MODE, clave);
            byte[] buffer;
            byte[] buffer_cifrado;
            int longitud;

            try
            {
                fs = new FileStream(archivo, FileMode.Open, FileAccess.Read);

                longitud = (int)fs.Length;
                buffer_cifrado = new byte[longitud];
                fs.Read(buffer_cifrado, 0, longitud);
                buffer = cipher.update(buffer_cifrado, 0, buffer_cifrado.Length);
                memStream.Write(buffer, 0, buffer.Length);
                memStream.Seek(0, SeekOrigin.Begin);
                lista = (ArrayList)(formatter.Deserialize(memStream));
                memStream.Close();
                fs.Close();
            }
            catch (FileNotFoundException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        // recupera la lista de un archivo sin cifrar
        internal Usuarios(string archivo)
        {
            try
            {
                FileStream fs = new FileStream(archivo, FileMode.Open, FileAccess.Read);
                BinaryFormatter formatter = new BinaryFormatter();

                lista = (ArrayList)formatter.Deserialize(fs);
                fs.Close();
            }
            catch (FileNotFoundException)
            {
            }
            catch (Exception)
            {
            }
        }

        //devuelve un vector de string con los nombres de usuario de los 
        //usuarios en la lista
        internal string[] nombresUsuarios()
        {
            string[] nombres = new string[lista.Count];
            int i = 0;

            foreach (Object o in lista)
            {
                nombres[i] = ((Usuario)o).Nombre;
                i++;
            }

            return nombres;
        }

        //devuelve el usuario en la posicin indicada
        internal Usuario buscarUsuario(int posicion)
        {
            int cuenta = 0;
            foreach (Object o in lista)
            {
                if (cuenta == posicion)
                    return (Usuario)o;
                cuenta++;
            }
            return null;
        }

        //devuelve un usuario cuyo nombre coincida con el especificado
        internal Usuario buscarUsuarioPorNombre(string nombre)
        {
            Usuario usuario;

            foreach (Object o in lista)
            {
                usuario = (Usuario)o;
                if (usuario.Nombre.Equals(nombre))
                    return usuario;
            }
            return null;
        }
        
        //devuelve un usuario cuyo email coincida con el especificado
        internal Usuario buscarUsuarioPorEmail(string email)
        {
            Usuario usuario;

            foreach (Object o in lista)
            {
                usuario = (Usuario)o;
                if (usuario.Email.Equals(email,StringComparison.CurrentCultureIgnoreCase))
                    return usuario;
            }
            return null;
        }

        //elimina un usuario cuyo nombre coincida con el indicado
        internal void eliminarUsuarioPorNombre(string nombre)
        {
            Usuario usuario;

            foreach (Object o in lista)
            {
                usuario = (Usuario)o;
                if (usuario.Nombre.Equals(nombre))
                {
                    lista.Remove(o);
                    return;
                }
            }
        }

        //elimina un usuario cuyo nombre coincida con el especificado
        internal void eliminarUsuarioPorEmail(string email)
        {
            Usuario usuario;

            foreach (Object o in lista)
            {
                usuario = (Usuario)o;
                if (usuario.Email.Equals(email))
                {
                    lista.Remove(o);
                    return;
                }
            }
        }

        //busca una clave pblica para unos determinados usuario y algoritmo
        internal byte[] buscarClavePublicaPorNombre(string nombre, string algoritmo)
        {
            Usuario usuario;

            foreach (Object o in lista)
            {
                usuario = (Usuario)o;
                if (usuario.Nombre.Equals(nombre))
                {
                    return usuario.buscarClavePublica(algoritmo);
                }
            }
            return null;
        }

        //busca una clave pblica para un usuario con el email indicado
        internal byte[] buscarClavePublicaPorEmail(string email, string algoritmo)
        {
            Usuario usuario;

            foreach (Object o in lista)
            {
                usuario = (Usuario)o;
                if (usuario.Email.Equals(email,StringComparison.CurrentCultureIgnoreCase))
                {
                    return usuario.buscarClavePublica(algoritmo);
                }
            }
            return null;
        }

        //elimina una clave pblica de unos determinados usuario y algoritmo
        internal void eliminarClavePublicaPorNombre(string nombre, string algoritmo)
        {
            Usuario usuario;

            foreach (Object o in lista)
            {
                usuario = (Usuario)o;
                if (usuario.Nombre.Equals(nombre))
                {
                    usuario.eliminarClavePublica(algoritmo);
                    return;
                }
            }
        }

        //elimina una clave pblica de un usuario cuyo email coincida con el indicado
        internal void eliminarClavePublicaPorEmail(string email, string algoritmo)
        {
            Usuario usuario;

            foreach (Object o in lista)
            {
                usuario = (Usuario)o;
                if (usuario.Email.Equals(email))
                {
                    usuario.eliminarClavePublica(algoritmo);
                    return;
                }
            }
        }

        //aade un usuario a la lista
        internal void aadirUsuario(Usuario usuario)
        {
            try
            {
                lista.Add(usuario);
            }
            catch (NotSupportedException e)
            {
                Console.WriteLine(e.Message);
            }
        }

        //aade un usuario a la lista
        internal void aadirUsuario(string nombre, string email, bool pleno, ArrayList claves)
        {
            try
            {
                lista.Add(new Usuario(nombre, email, pleno, claves));
            }
            catch (NotSupportedException e)
            {
                Console.WriteLine(e.Message);
            }
        }

        //aade un usuario a la lista
        internal void aadirUsuario(string nombre, string email, bool pleno, params ParClavePublicaNombreAlgoritmo[] claves)
        {
            try
            {
                lista.Add(new Usuario(nombre, email, pleno, claves));
            }
            catch (NotSupportedException e)
            {
                Console.WriteLine(e.Message);
            }
        }

        // devuelve true si la operacin tuvo xito; false si hubo un error
        // si no se encuentra el archivo no se considera un error, se crea uno nuevo, pero
        // se informa al usuario, por si ste cree que hay un problema de seguridad
        internal Boolean guardarEnArchivo(string archivo)
        {
            FileStream fs;
            try
            {
                fs = new FileStream(archivo, FileMode.Create, FileAccess.Write);
                try
                {
                    BinaryFormatter formatter = new BinaryFormatter();
                    formatter.Serialize(fs, lista);
                    fs.Close();
                    return true;
                }
                catch (Exception e)
                {
                    Console.WriteLine(e.Message);
                    fs.Close();
                    return false;
                }
            }
            catch (FileNotFoundException)
            {
                fs = new FileStream(archivo, FileMode.OpenOrCreate, FileAccess.Write);
                try
                {
                    BinaryFormatter formatter = new BinaryFormatter();
                    formatter.Serialize(fs, lista);
                    fs.Close();
                    return true;
                }
                catch (Exception)
                {
                    fs.Close();
                    return false;
                }
            }
        }

        //guarda la lista en un archivo cifrado
        internal Boolean guardarEnArchivoCifrado(String archivo, string algoritmo, SecretKey claveSimetrica)
        {
            FileStream fs = null;
            MemoryStream memStream = new MemoryStream();
            BinaryFormatter formatter = new BinaryFormatter();
            byte[] buffer;
            byte[] buffer_cifrado;
            //inicializamos el cifrador
            Cipher desCipher = Cipher.getInstance(algoritmo);
            desCipher.init(Cipher.ENCRYPT_MODE, claveSimetrica);

            try
            {
                //copiamos la lista a memStream
                formatter.Serialize(memStream, lista);
                int longitud = (int)memStream.Length;
                buffer = new byte[longitud];

                //leemos la lista en forma de byte[], ahora est almacenada en buffer
                memStream.Seek(0, SeekOrigin.Begin);
                memStream.Read(buffer, 0, longitud);
                //abrimos el archivo sobreescribiendo el anterior
                fs = new FileStream(archivo, FileMode.Create, FileAccess.Write);
                //ciframos buffer y escribimos el resultado en el archivo
                buffer_cifrado = desCipher.doFinal(buffer);
                fs.Write(buffer_cifrado, 0, buffer_cifrado.Length);
                fs.Close();
                memStream.Close();
                return true;
            }
            catch (Exception e)
            {
                MessageBox.Show("Error al escribir el archivo de usuarios: " + e.Message);
                return false;
            }
            finally
            {
                fs.Close();
                memStream.Close();
            }
        }        

        //devuelve el tamao de la lista
        internal int tamao()
        {
            return lista.Count;
        }
    }
}
